package com.emall.flash.api.controller.shop;

import com.emall.flash.bean.constant.factory.PageFactory;
import com.emall.flash.bean.core.BussinessLog;
import com.emall.flash.bean.entity.shop.ActivityInfo;
import com.emall.flash.bean.entity.system.User;
import com.emall.flash.bean.exception.ApplicationException;
import com.emall.flash.bean.exception.ApplicationExceptionEnum;
import com.emall.flash.bean.vo.front.Rets;
import com.emall.flash.bean.vo.query.SearchFilter;
import com.emall.flash.cache.CacheDao;
import com.emall.flash.security.JwtUtil;
import com.emall.flash.service.shop.ActivityService;
import com.emall.flash.service.system.ManagerService;
import com.emall.flash.utils.RedisCons;
import com.emall.flash.utils.TimeUtil;
import com.emall.flash.utils.factory.Page;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.redisson.api.RLock;
import org.redisson.api.RedissonClient;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Sort;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.Calendar;
import java.util.Date;
import java.util.Set;
import java.util.concurrent.TimeUnit;


@RestController
@RequestMapping("/shop/activity")
public class ActivityController {
    private Logger logger = LoggerFactory.getLogger(getClass());
    @Autowired
    private ActivityService activityService;
    @Autowired
    private ManagerService managerService;
    @Autowired
    private CacheDao cacheDao;
    @Autowired
    private RedissonClient redissonClient;

    @Resource
    private RedisTemplate<String, Object> redisTemplate;

    /**
     * 获取所有活动
     */
    @RequestMapping(value = "/list",method = RequestMethod.GET)
    public Object list(@RequestParam(required = false) Long tenantId,
                       @RequestParam(required = false) String name) {
        Page<ActivityInfo> page = new PageFactory().defaultPage();
        User loginUser = managerService.get(JwtUtil.getUserId());
        if (loginUser.getTenantId()!=null)
            page.addFilter(SearchFilter.build("tenantId", loginUser.getTenantId()));
        else if (tenantId!=null)
            page.addFilter(SearchFilter.build("tenantId", tenantId));
        if (StringUtils.isNotBlank(name))
            page.addFilter(SearchFilter.build("name", SearchFilter.Operator.LIKE, name));
        page.addFilter(SearchFilter.build("delFlag", 0));
        page.setSort(Sort.by(Sort.Direction.DESC, "createTime"));
        page = activityService.queryPage(page);
        for (ActivityInfo info : page.getRecords()) {
            Integer number  = info.getNumber();
            Integer viewNum = cacheDao.hget(RedisCons.ACTIVITY_COUNT , info.getId(),Integer.class);
            if (null != viewNum)
                number = viewNum;
            info.setEntryNum(info.getNumber() - number);
        }
        return Rets.success(page);
    }

    /**
     * 获取所有活动
     */
    @RequestMapping(value = "/getById",method = RequestMethod.GET)
    public Object getById(@RequestParam Long id) {
        return Rets.success(activityService.get(id));
    }

    @RequestMapping(value = "/add",method = RequestMethod.POST)
    @BussinessLog(value = "添加活动", key = "id")
    public Object add(@RequestBody ActivityInfo activityInfo) {
        activityInfo.setDelFlag(0);
        activityInfo.setStatus(0);
        if (Boolean.TRUE.equals(activityService.isRepeat(activityInfo)))
            return Rets.failure("活动名称重复");
        User loginUser = managerService.get(JwtUtil.getUserId());
        activityInfo.setTenantId(loginUser.getTenantId());
        activityService.insert(activityInfo);
        return Rets.success(activityInfo.getId());
    }

    @RequestMapping(value = "/update",method = RequestMethod.PUT)
    @BussinessLog(value = "修改活动", key = "id")
    public Object update(@RequestBody ActivityInfo activityInfo) {
        if (Boolean.TRUE.equals(activityService.isRepeat(activityInfo)))
            return Rets.failure("活动名称重复");
        activityService.update(activityInfo);
        return Rets.success();
    }


    @RequestMapping(method = RequestMethod.DELETE)
    @BussinessLog(value = "删除活动", key = "id")
    public Object delete(@RequestParam Long id) {
        ActivityInfo activityInfo = activityService.get(id);
        activityInfo.setDelFlag(1);
        activityService.update(activityInfo);
        return Rets.success();
    }

    @RequestMapping(value = "/updateStatus",method = RequestMethod.PUT)
    @BussinessLog(value = "发布活动", key = "id")
    public Object updateStatus(@RequestBody ActivityInfo activityInfo) {
        ActivityInfo info = activityService.get(activityInfo.getId());
        if (activityInfo.getStatus().equals(1)) {
            if (info.getStatus() != 0)
                throw new ApplicationException(ApplicationExceptionEnum.ACTIVITY_PUBLISH_ERROR);
            info.setStatus(activityInfo.getStatus());
            activityService.update(info);
            Date endTime = TimeUtil.getTime(info.getActivityEndDate(), Calendar.DATE,1);
            //将活动信息插入redis
            long time = TimeUtil.getTimeSecond(new Date(),endTime);
            cacheDao.set(RedisCons.ACTIVITY_DETAIL +":"+ activityInfo.getId(),info,time);
            cacheDao.set(RedisCons.ACTIVITY_COUNT +":"+ activityInfo.getId(),info.getNumber(),time);
        } else {
            //如果取消发布，加redis锁
            boolean isGetLock = false;
            RLock lock = redissonClient.getLock(RedisCons.ACTIVITY_INFO + activityInfo.getId());
            try {
                isGetLock = lock.tryLock(Long.parseLong(RedisCons.ORDER_INFO_LOCK_WAIT_TIME),
                        Long.parseLong(RedisCons.ORDER_INFO_LOCK_LEASE_TIME), TimeUnit.SECONDS);
                if (isGetLock) {
                    //校验活动是否已经报名，状态是否为发布
                    Set<String> map = redisTemplate.keys(RedisCons.ORDER_ACTIVITY + ":" + activityInfo.getId() + "*");
                    if (CollectionUtils.isNotEmpty(map)) //报名人数和实际剩余报名人数不等
                        throw new ApplicationException(ApplicationExceptionEnum.ACTIVITY_STATUS_ERROR);
                    if (info.getStatus() == 0)
                        throw new ApplicationException(ApplicationExceptionEnum.ACTIVITY_CANCEL_ERROR);
                    info.setStatus(activityInfo.getStatus());
                    activityService.update(info);
                    cacheDao.hdel(RedisCons.ACTIVITY_DETAIL , activityInfo.getId());
                    cacheDao.hdel(RedisCons.ACTIVITY_COUNT , activityInfo.getId());
                }
            } catch (InterruptedException e) {
                Thread.currentThread().interrupt();
                isGetLock = false;
            } finally {
                lock.unlock();
                logger.error("释放成功");
            }
        }

        return Rets.success();
    }

}
