package com.emall.flash.api.controller.system;

import com.emall.flash.bean.constant.Const;
import com.emall.flash.bean.constant.factory.PageFactory;
import com.emall.flash.bean.constant.state.ManagerStatus;
import com.emall.flash.bean.core.BussinessLog;
import com.emall.flash.bean.dto.UserDto;
import com.emall.flash.bean.entity.shop.Goods;
import com.emall.flash.bean.entity.shop.Tenant;
import com.emall.flash.bean.entity.system.User;
import com.emall.flash.bean.enumeration.Permission;
import com.emall.flash.bean.exception.ApplicationException;
import com.emall.flash.bean.exception.ApplicationExceptionEnum;
import com.emall.flash.bean.vo.front.Rets;
import com.emall.flash.bean.vo.query.SearchFilter;
import com.emall.flash.core.factory.UserFactory;
import com.emall.flash.security.JwtUtil;
import com.emall.flash.service.shop.TenantService;
import com.emall.flash.service.system.LogObjectHolder;
import com.emall.flash.service.system.ManagerService;
import com.emall.flash.utils.*;
import com.emall.flash.utils.factory.Page;
import com.emall.flash.warpper.UserWarpper;
import com.emall.flash.web.controller.BaseController;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;
import java.util.Arrays;
import java.util.List;

/**
 * UserController
 *
 * @author enilu
 * @version 2018/9/15 0015
 */
@RestController
@RequestMapping("/user")
public class UserController extends BaseController {
    @Autowired
    private ManagerService managerService;
    @Value("${init.shopAdminRoleId}")
    private String shopAdminRoleId;
    @Autowired
    private TenantService tenantService;

    @RequestMapping(value = "/list",method = RequestMethod.GET)
    @RequiresPermissions(value = {Permission.USER})
    public Object list(@RequestParam(required = false) String account,
                       @RequestParam(required = false) String name,
                       @RequestParam(required = false) Integer sex,
                       @RequestParam(required = false) List<Long> tenantIds){
        Page<User> page = new PageFactory().defaultPage();
        if(StringUtil.isNotEmpty(name)){
            page.addFilter(SearchFilter.build("name", SearchFilter.Operator.LIKE, name));
        }
        if(StringUtil.isNotEmpty(account)){
            page.addFilter(SearchFilter.build("account", SearchFilter.Operator.LIKE, account));
        }
        User loginUser = managerService.get(JwtUtil.getUserId());
        if (loginUser.getTenantId()!=null)
            page.addFilter(SearchFilter.build("tenantId", SearchFilter.Operator.EQ, loginUser.getTenantId()));
        else if (tenantIds!=null && !tenantIds.isEmpty())
            page.addFilter(SearchFilter.build("tenantId", SearchFilter.Operator.IN, tenantIds));
        page.addFilter( "status",SearchFilter.Operator.IN, Lists.newArrayList(1,2));
        page.addFilter("sex", sex);
        page = managerService.queryPage(page);
        for (User record : page.getRecords()) {
            if (record.getTenantId()!=null) {
                Tenant tenant = tenantService.get(record.getTenantId());
                record.setTenantName(tenant.getName());
            }
        }
        List list = (List) new UserWarpper(BeanUtil.objectsToMaps(page.getRecords())).warp();
        page.setRecords(list);
        return Rets.success(page);
    }
    @RequestMapping(method = RequestMethod.POST)
    @BussinessLog(value = "编辑管理员", key = "name")
    @RequiresPermissions(value = {Permission.USER_EDIT})
    public Object save( @Valid UserDto user){
        if(user.getId()==null) {
            // 判断账号是否重复
            User theUser = managerService.findByAccount(user.getAccount());
            if (theUser != null) {
                throw new ApplicationException(ApplicationExceptionEnum.USER_ALREADY_REG);
            }
            if(user.getPassword()==null){
                return Rets.failure("密码不能为空");
            }
            // 完善账号信息
            user.setSalt(RandomUtil.getRandomString(5));
            user.setPassword(MD5.md5(user.getPassword(), user.getSalt()));
            user.setStatus(ManagerStatus.OK.getCode());
            User loginUser = managerService.get(JwtUtil.getUserId());
            User insertUser = UserFactory.createUser(user, new User());
            insertUser.setTenantId(loginUser.getTenantId());
            if (loginUser.getUserType().equals(1))
                insertUser.setUserType(2);
            managerService.insert(insertUser);
        }else{

            User oldUser = managerService.get(user.getId());
            LogObjectHolder.me().set(oldUser);
            managerService.update(UserFactory.updateUser(user,oldUser));
        }
        return Rets.success();
    }

    @BussinessLog(value = "删除管理员", key = "userId")
    @RequestMapping(method = RequestMethod.DELETE)
    @RequiresPermissions(value = {Permission.USER_DEL})
    public Object remove(@RequestParam Long userId){
        if (userId == null) {
            throw new ApplicationException(ApplicationExceptionEnum.REQUEST_NULL);
        }
        if(userId.intValue()<=2){
            return Rets.failure("不能删除初始用户");
        }
        User user = managerService.get(userId);
        user.setStatus(ManagerStatus.DELETED.getCode());
        managerService.update(user);
        return Rets.success();
    }
    @BussinessLog(value="设置用户角色",key="userId")
    @RequestMapping(value="/setRole",method = RequestMethod.GET)
    @RequiresPermissions(value = {Permission.USER_EDIT})
    public Object setRole(@RequestParam("userId") Long userId, @RequestParam("roleIds") String roleIds) {
        if (BeanUtil.isOneEmpty(userId, roleIds)) {
            throw new ApplicationException(ApplicationExceptionEnum.REQUEST_NULL);
        }
        //商圈管理员不能分配商圈管理员和超级管理员角色
        User loginUser = managerService.get(JwtUtil.getUserId());
        String[] roleId = roleIds.split(",");
        if (loginUser.getUserType().equals(1) && (Arrays.asList(roleId).contains(shopAdminRoleId))) {
            return Rets.failure("不能分配该角色");
        }
        //不能修改超级管理员
        if (userId.equals(Const.ADMIN_ID)) {
            throw new ApplicationException(ApplicationExceptionEnum.CANT_CHANGE_ADMIN);
        }
        User user = managerService.get(userId);
        user.setRoleid(roleIds);
        managerService.update(user);
        return Rets.success();
    }

}
